# -*- coding: utf-8 -*-

from .agent.api import DEFAULT_USERNAME
from .agent.api import DEFAULT_PASSWORD
from .agent.api import DEFAULT_HOST
from .agent.api import DEFAULT_PORT
from .agent.api import SOAPRequest
from .agent.api import SOAPResponse
from .sdk.api import run_soap_command
from .sdk.api import canned
